import numpy as np
import sys
import networkx as nx
import re
import pandas as pd

def preprocess(hypernymy_ds_file, processed_ds_file):
    '''
    This function recieves two arguments: the input file to preprocess and the output destination file.
    It preprocess the hypernymy dataset provided
    '''
    dropped_hypernyms = '(.*?thing_|.*?name_|.*?example_|.*?activity_|_problem_|_attempt_|_condition_|_process_|_feature_|_mistake_|_session_|_change_|_message_|_classic_|_object_|_stretch_|_return_|_factor_|_issue_|_option_|_other_|_act_|.*?item_|.*?place.*?|_diverse_|_allergen_)'
    dfin = pd.read_csv(hypernymy_ds_file, names=['hyponym', 'hypernym', 'score'], sep=';|\t', engine='python')
    # drop too general hypernyms
    drop_filter = dfin['hypernym'].str.contains(dropped_hypernyms)
    dfout = dfin[~drop_filter]
    # replace space separators
    dfout['hyponym'] = dfout['hyponym'].str.replace('_', ' ')
    dfout['hyponym'] = dfout['hyponym'].str.replace('+', ' ')
    dfout['hypernym'] = dfout['hypernym'].str.replace('_', ' ')
    dfout['hypernym'] = dfout['hypernym'].str.replace('+', ' ')
    # replace html encoding
    dfout['hyponym'] = dfout['hyponym'].str.replace('%27', "'")
    dfout['hypernym'] = dfout['hypernym'].str.replace('%27', "'")
    dfout['hyponym'] = dfout['hyponym'].str.replace('%B1', "+")
    dfout['hypernym'] = dfout['hypernym'].str.replace('%B1', "+")
    # dfout['hyponym'] = dfout['hyponym'].str.replace('%2F', "/")
    # dfout['hypernym'] = dfout['hypernym'].str.replace('%2F', "/")
    # strip
    dfout['hyponym'] = dfout['hyponym'].str.strip()
    dfout['hypernym'] = dfout['hypernym'].str.strip()
    
    # save processed output file
    dfout.to_csv(processed_ds_file, header=False, index=False)  
    #np.savetxt(processed_ds_file, dfout, fmt='%s', delimiter='\t')
   
'''
main function
'''
if __name__ == '__main__':
    if len(sys.argv) != 3:
        raise ValueError('Expected exactly 2 argument: hypernymy dataset file and output processed dataset file')
    preprocess(sys.argv[1], sys.argv[2])