import sys
import pickle
import numpy as np
from nltk.tree import Tree
from collections import defaultdict

def find_fathers(words,conf_data_set,fathers_dict):
    """Use the clean dataset that contains rules with scores higher than 0.55 
    to build a dictionary of nodes and their super classes
    input words has the priority to be super classes """
    for word in words:
        rules = conf_data_set[word] # each element in conf_data_set has shape: lhs:(rhs,score) 
        if len(rules) > 0 :
            candidates = []
            for rule in rules:
                for token in words:
                    if token in rule[0].split(): # if the super class of the current word is among the input then add it as its father
                        candidates.append((token,rule[1]))
            if len(candidates) > 0:
                    index = np.argmax([score for _,score in candidates])
                    winner = candidates[index][0]
                    if winner == word:
                        fathers_dict[word]
                    else:
                         if word not in fathers_dict[winner]:
                                fathers_dict[winner].append(word)
            else: # if no fathers were found among input words get a father from the dataser
                index = np.argmax([score for _,score in rules])
                winner = rules[index][0]
                if winner == word:
                    fathers_dict[word]
                else:
                    if word not in fathers_dict[winner]:
                                fathers_dict[winner].append(word)
        else:   
            fathers_dict[word]
            
    # return fathers_dict

def build_tree(node,output_dict):
    """Build nltk tree recursivly from a dictionary"""
    if len(output_dict[node]) == 0:
        return (node)
    else:
        taxo = Tree(node,[build_tree(child,output_dict) for child in output_dict[node]])
        return taxo

def build_taxonomy(input_file):
    ## read preprocessed data 
    with open('clean_dataset.pkl', 'rb') as f:
        clean_data = pickle.load(f)
    
    ## read input words
    with open(input_file,'r') as f:
        input_words = f.readlines()
    
    ## find parent of each word
    input_words = [word.strip('\n') for word in input_words]
    fathers_dictt = defaultdict(list)
    find_fathers(input_words,clean_data,fathers_dictt)
    
    ## repeat for the founded parents
    input_words = list(fathers_dictt.keys())
    find_fathers(input_words,clean_data,fathers_dictt)
    
    ## organize children and parents in dict
    children = [child for value in fathers_dictt.values() for child in value ]
    root_children = [key for key in fathers_dictt.keys() if key not in children ]
    fathers_dictt['entity'] = root_children

    ## build taxonomy from fathers dict
    taxonomy = build_tree('entity',fathers_dictt)

    ## draw taxonomy as tree
    tree = taxonomy
    tree.draw()

'''
main function
'''
if __name__ == '__main__':
    if len(sys.argv) != 2:
        raise ValueError('Expected exactly 1 argument: input file')
    build_taxonomy(sys.argv[1])
