import networkx as nx


class PathNode:
    def __init__(self, word: str):
        self.word = word
        self.children = list['PathNode']()

    def __repr__(self):
        return self.word

    def get_word(self):
        return self.word

    def set_children(self, children):
        self.children = children

    def get_children(self):
        return self.children

    def get_all_children(self):
        children = list()
        children.extend(self.get_children())
        for child in self.get_children():
            children.extend(child.get_all_children())
        return children

    def count_nodes(self):
        return len(self.get_all_children()) + 1

    def get_min_depth(self):
        if len(self.get_children()) == 0:
            return 1
        return min(map(lambda x: x.get_min_depth(), self.get_children())) + 1

    def get_max_depth(self):
        if len(self.get_children()) == 0:
            return 1
        return max(map(lambda x: x.get_max_depth(), self.get_children())) + 1

    def truncate_one(self, min_depth, vital_words, parent_depth=0):
        trunc_child_candidates = list()
        trunc_candidates = list()
        for child in self.get_children():
            is_remove_candidate = True
            if parent_depth + 1 + child.get_min_depth() <= min_depth:
                is_remove_candidate = False
            if child.get_word() in vital_words:
                is_remove_candidate = False
            if len(child.get_children()) > 1:
                is_remove_candidate = False
            if is_remove_candidate:
                trunc_candidates.append(child)

            if child.get_max_depth() + parent_depth > min_depth:
                trunc_child_candidates.append(child)

        trunc_candidates = sorted(trunc_candidates, key=lambda x: x.get_max_depth(), reverse=True)
        if len(trunc_candidates) != 0:
            trunc_candidate = trunc_candidates[0]
            self.children.remove(trunc_candidate)
            self.children.extend(trunc_candidate.get_children())
            return True

        trunc_child_candidates = sorted(trunc_child_candidates, key=lambda x: x.get_max_depth(), reverse=True)
        if len(trunc_child_candidates) == 0:
            return False

        for candidate in trunc_child_candidates:
            if candidate.truncate_one(min_depth, vital_words, parent_depth + 1):
                return True
        return False





    def plot_tree(self):
        graph = nx.Graph()
        graph.add_node(self)
        for node in self.get_children():
            self._plot_tree_help(graph, self, node)
        return graph

    def _plot_tree_help(self, graph, parent: 'PathNode', child: 'PathNode'):
        graph.add_node(child)
        graph.add_edge(parent, child)
        for node in child.get_children():
            self._plot_tree_help(graph, child, node)


class TreeGenerator:
    root_node_word = "entity"

    def __init__(self, word_paths: dict[str, list[list[str]]]):
        self.word_paths = word_paths

    def generate(self):
        for vital_word, paths in self.word_paths.items():
            for path in paths:
                if len(path) == 0:
                    continue
                if path[0] == self.root_node_word:
                    del path[0]

        root_node = PathNode(self.root_node_word)
        self._generate_rec(self.word_paths, root_node)
        return root_node

    def _generate_rec(self, word_paths, parent):
        # Group by words and count them for current index max once per vital_word
        child_nodes = {}
        while len(word_paths.keys()) > 0:
            index_path_words = self.group_by_count_desc(word_paths)
            index_path_words_keys = list(index_path_words.keys())
            if len(index_path_words_keys) == 0:
                break

            current_common_word = index_path_words_keys[0]
            current_child_word_paths = {}
            for vital_word, paths in list(word_paths.items()):
                if len(paths) == 0:
                    continue
                relevant_paths = []
                word_included = False
                for path in paths:
                    if len(path) == 0:
                        continue
                    word = path[0]
                    if word == current_common_word:
                        relevant_paths.append(path)
                        word_included = True
                if word_included:
                    del word_paths[vital_word]
                    current_child_word_paths[vital_word] = relevant_paths
            for paths in current_child_word_paths.values():
                for path in paths:
                    if len(path) != 0:
                        del path[0]
            child_nodes[PathNode(current_common_word)] = current_child_word_paths
        parent.set_children(list(child_nodes.keys()))
        for child_node, paths in child_nodes.items():
            self._generate_rec(paths, child_node)

    def group_by_count_desc(self, word_paths):
        index_path_words = {}
        for vital_word, paths in word_paths.items():
            if len(paths) == 0:
                continue
            seen_words = set()
            for path in paths:
                if len(path) == 0:
                    continue
                word = path[0]
                if word in seen_words:
                    continue
                seen_words.add(word)
                if word not in index_path_words.keys():
                    index_path_words[word] = 0
                index_path_words[word] = index_path_words[word] + 1
        return {k: v for k, v in sorted(index_path_words.items(), key=lambda item: item[1], reverse=True)}
